package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type CarInfoController struct {
	api.Api
}

// GetPage 获取车辆信息列表
// @Summary 获取车辆信息列表
// @Description 获取车辆信息列表
// @Tags 车辆信息
// @Param carNo query string false "车牌号"
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.CarInfo}} "{"code": 200, "data": [...]}"
// @Router /api/car-info [get]
// @Security Bearer
func (e CarInfoController) GetPage(c *gin.Context) {
	s := service.CarInfo{}
	req := dto.CarInfoGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	list := make([]model.CarInfo, 0)
	var count int64

	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取车辆信息
// @Summary 通过id获取车辆信息
// @Description 通过id获取车辆信息
// @Tags 车辆信息
// @Param id path string true "车辆信息id"
// @Success 200 {object} response.Response{data=model.CarInfo} "{"code": 200, "data": [...]}"
// @Router /api/car-info/{id} [get]
// @Security Bearer
func (e CarInfoController) Get(c *gin.Context) {
	s := service.CarInfo{}
	req := dto.CarInfoGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.CarInfo
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加车辆信息
// @Summary 添加车辆信息
// @Description 添加车辆信息
// @Tags 车辆信息
// @Accept  application/json
// @Product application/json
// @Param data body dto.CarInfoInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/car-info [post]
// @Security Bearer
func (e CarInfoController) Insert(c *gin.Context) {
	s := service.CarInfo{}
	req := dto.CarInfoInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)
	// 获取运输企业code
	truck, err := model.GetProvCodeById(p.DeptId)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	req.SetCmpCode(truck.CmpCode)
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改车辆信息
// @Summary 修改车辆信息
// @Description 修改车辆信息
// @Tags 车辆信息
// @Accept  application/json
// @Product application/json
// @Param data body dto.CarInfoUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/car-info [put]
// @Security Bearer
func (e CarInfoController) Update(c *gin.Context) {
	s := service.CarInfo{}
	req := dto.CarInfoUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除车辆信息
// @Summary 删除车辆信息
// @Description 删除车辆信息
// @Tags 车辆信息
// @Accept  application/json
// @Product application/json
// @Param data body dto.CarInfoDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/car-info [delete]
// @Security Bearer
func (e CarInfoController) Delete(c *gin.Context) {
	s := service.CarInfo{}
	req := dto.CarInfoDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}

// BindTruckUser 绑定货车司机
// @Summary 绑定货车司机
// @Description 绑定货车司机
// @Tags 车辆信息
// @Accept  application/json
// @Product application/json
// @Param data body dto.CarInfoInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/car-info/bind-truck-user [post]
// @Security Bearer
func (e CarInfoController) BindTruckUser(c *gin.Context) {
	s := service.CarInfo{}
	req := dto.CarInfoBindTruckUserReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	err = s.BindTruckUser(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.CarNo, "绑定成功")
}
