package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type DeviceController struct {
	api.Api
}

// GetPage 获取设备列表
// @Summary 获取设备列表
// @Description 获取设备列表
// @Tags 设备
// @Param sn query string false "设备编码"
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.Device}} "{"code": 200, "data": [...]}"
// @Router /api/device [get]
// @Security Bearer
func (e DeviceController) GetPage(c *gin.Context) {
	s := service.Device{}
	req := dto.DeviceGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	list := make([]model.Device, 0)
	var count int64

	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取设备
// @Summary 通过id获取设备
// @Description 通过id获取设备
// @Tags 设备
// @Param id path string true "设备id"
// @Success 200 {object} response.Response{data=model.Device} "{"code": 200, "data": [...]}"
// @Router /api/device/{id} [get]
// @Security Bearer
func (e DeviceController) Get(c *gin.Context) {
	s := service.Device{}
	req := dto.DeviceGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.Device
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加设备
// @Summary 添加设备
// @Description 添加设备
// @Tags 设备
// @Accept  application/json
// @Product application/json
// @Param data body dto.DeviceInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/device [post]
// @Security Bearer
func (e DeviceController) Insert(c *gin.Context) {
	s := service.Device{}
	userSvc := service.SysUser{}
	req := dto.DeviceInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)
	var userObj model.SysUser
	err = userSvc.GetByProvUserId(req.ProvUserId, &userObj)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.ProvUserId = userObj.ProvUserId
	req.ProvCmpCode = userObj.ProvUser.CmpCode
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改设备
// @Summary 修改设备
// @Description 修改设备
// @Tags 设备
// @Accept  application/json
// @Product application/json
// @Param data body dto.DeviceUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/device [put]
// @Security Bearer
func (e DeviceController) Update(c *gin.Context) {
	s := service.Device{}
	userSvc := service.SysUser{}
	req := dto.DeviceUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除设备
// @Summary 删除设备
// @Description 删除设备
// @Tags 设备
// @Accept  application/json
// @Product application/json
// @Param data body dto.DeviceDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/device [delete]
// @Security Bearer
func (e DeviceController) Delete(c *gin.Context) {
	s := service.Device{}
	req := dto.DeviceDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}
