package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type DispatchCostController struct {
	api.Api
}

// GetPage 获取派费列表
// @Summary 获取派费列表
// @Description 获取派费列表
// @Tags 派费
// @Param name query string false "派费名称"
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.DispatchCost}} "{"code": 200, "data": [...]}"
// @Router /api/dispatch-cost [get]
// @Security Bearer
func (e DispatchCostController) GetPage(c *gin.Context) {
	s := service.DispatchCost{}
	req := dto.DispatchCostGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	list := make([]model.DispatchCost, 0)
	var count int64

	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取派费
// @Summary 通过id获取派费
// @Description 通过id获取派费
// @Tags 派费
// @Param id path string true "派费id"
// @Success 200 {object} response.Response{data=model.DispatchCost} "{"code": 200, "data": [...]}"
// @Router /api/dispatch-cost/{id} [get]
// @Security Bearer
func (e DispatchCostController) Get(c *gin.Context) {
	s := service.DispatchCost{}
	req := dto.DispatchCostGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.DispatchCost
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加派费
// @Summary 添加派费
// @Description 添加派费
// @Tags 派费
// @Accept  application/json
// @Product application/json
// @Param data body dto.DispatchCostInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/dispatch-cost [post]
// @Security Bearer
func (e DispatchCostController) Insert(c *gin.Context) {
	s := service.DispatchCost{}
	req := dto.DispatchCostInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改派费
// @Summary 修改派费
// @Description 修改派费
// @Tags 派费
// @Accept  application/json
// @Product application/json
// @Param data body dto.DispatchCostUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/dispatch-cost [put]
// @Security Bearer
func (e DispatchCostController) Update(c *gin.Context) {
	s := service.DispatchCost{}
	req := dto.DispatchCostUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除派费
// @Summary 删除派费
// @Description 删除派费
// @Tags 派费
// @Accept  application/json
// @Product application/json
// @Param data body dto.DispatchCostDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/dispatch-cost [delete]
// @Security Bearer
func (e DispatchCostController) Delete(c *gin.Context) {
	s := service.DispatchCost{}
	req := dto.DispatchCostDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}
