package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type FillGunController struct {
	api.Api
}

// GetPage 获取充装枪列表
// @Summary 获取充装枪列表
// @Description 获取充装枪列表
// @Tags 充装枪
// @Param GunCode     query string false "充装枪编码"
// @Param ScanGunCode query string false "扫描枪编码"
// @Param PersonCode  query string false "充装人员编码"
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.FillGun}} "{"code": 200, "data": [...]}"
// @Router /api/goods [get]
// @Security Bearer
func (e FillGunController) GetPage(c *gin.Context) {
	s := service.FillGun{}
	req := dto.FillGunGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	list := make([]model.FillGun, 0)
	var count int64

	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取充装枪
// @Summary 通过id获取充装枪
// @Description 通过id获取充装枪
// @Tags 充装枪
// @Param id path string true "充装枪id"
// @Success 200 {object} response.Response{data=model.FillGun} "{"code": 200, "data": [...]}"
// @Router /api/goods/{id} [get]
// @Security Bearer
func (e FillGunController) Get(c *gin.Context) {
	s := service.FillGun{}
	req := dto.FillGunGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.FillGun
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加充装枪
// @Summary 添加充装枪
// @Description 添加充装枪
// @Tags 充装枪
// @Accept  application/json
// @Product application/json
// @Param data body dto.FillGunInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/goods [post]
// @Security Bearer
func (e FillGunController) Insert(c *gin.Context) {
	s := service.FillGun{}
	req := dto.FillGunInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改充装枪
// @Summary 修改充装枪
// @Description 修改充装枪
// @Tags 充装枪
// @Accept  application/json
// @Product application/json
// @Param data body dto.FillGunUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/goods [put]
// @Security Bearer
func (e FillGunController) Update(c *gin.Context) {
	s := service.FillGun{}
	req := dto.FillGunUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除充装枪
// @Summary 删除充装枪
// @Description 删除充装枪
// @Tags 充装枪
// @Accept  application/json
// @Product application/json
// @Param data body dto.FillGunDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/goods [delete]
// @Security Bearer
func (e FillGunController) Delete(c *gin.Context) {
	s := service.FillGun{}
	req := dto.FillGunDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}
