package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
)

type OrderController struct {
	api.Api
}

// GetPage 获取订单列表
// @Summary 获取订单列表
// @Description 获取订单列表
// @Tags 订单
// @Param state          query int false "订单状态 1-已下单 2-已派送 3-已送达 4-已取消"
// @Param source         query int false "订单状态 1-坐席下单 2-小程序"
// @Param orderStartTime query string false "下单开始时间"
// @Param orderEndTime   query string false "下单结束时间"
// @Param phone          query string false "客户电话"
// @Param customerId     query string false "客户id"
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.Order}} "{"code": 200, "data": [...]}"
// @Router /api/Order [get]
// @Security Bearer
func (e OrderController) GetPage(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	list := make([]model.Order, 0)
	var count int64

	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取下单
// @Summary 通过id获取下单
// @Description 通过id获取下单
// @Tags 订单
// @Param id path string true "下单id"
// @Success 200 {object} response.Response{data=model.Order} "{"code": 200, "data": [...]}"
// @Router /api/Order/{id} [get]
// @Security Bearer
func (e OrderController) Get(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.Order
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 下单
// @Summary 下单
// @Description 下单
// @Tags 订单
// @Accept  application/json
// @Product application/json
// @Param data body dto.OrderInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/order [post]
// @Security Bearer
func (e OrderController) Insert(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改订单
// @Summary 修改订单
// @Description 修改订单
// @Tags 订单
// @Accept  application/json
// @Product application/json
// @Param data body dto.OrderUpdateReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/order [post]
// @Security Bearer
func (e OrderController) Update(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "修改成功")
}

// Cancel 取消订单
// @Summary 取消订单
// @Description 取消订单
// @Tags 订单
// @Accept  application/json
// @Product application/json
// @Param data body dto.OrderCancelReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/order/cancel [post]
// @Security Bearer
func (e OrderController) Cancel(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderCancelReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	// 设置创建人
	req.SetUpdateBy(user.GetUserId(c))
	err = s.Cancel(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "取消成功")
}

// Delivery 派单
// @Summary 派单
// @Description 派单
// @Tags 订单
// @Accept  application/json
// @Product application/json
// @Param data body dto.OrderDeliveryReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/order [put]
// @Security Bearer
func (e OrderController) Delivery(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderDeliveryReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Delivery(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "派单成功")
}

// GetDeliveryPage 获取送气员订单列表
// @Summary 获取送气员订单列表
// @Description 获取送气员订单列表
// @Tags 订单
// @Success 200 {object} response.Response{data=response.Page{list=[]model.Order}} "{"code": 200, "data": [...]}"
// @Router /api/order/delivery [get]
// @Security Bearer
func (e OrderController) GetDeliveryPage(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderGetDeliveryPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	list := make([]model.Order, 0)
	var count int64

	err = s.GetDeliveryPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// UpdateState 修改订单状态
// @Summary 修改订单状态
// @Description 修改订单状态
// @Tags 订单
// @Accept  application/json
// @Product application/json
// @Param data body dto.OrderUpdateStateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/order/state [put]
// @Security Bearer
func (e OrderController) UpdateState(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderUpdateStateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.UpdateState(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "修改成功")
}

// Delete 删除
// @Summary 删除
// @Description 删除
// @Tags 订单
// @Accept  application/json
// @Product application/json
// @Param data body dto.OrderDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/order [put]
// @Security Bearer
func (e OrderController) Delete(c *gin.Context) {
	s := service.Order{}
	req := dto.OrderDeleteReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}
