package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
	"gogs.baozhida.cn/zoie/OAuth-core/sdk/config"
	"strconv"
	"strings"
)

type StoreController struct {
	api.Api
}

// GetPage 获取销售门店列表
// @Summary 获取销售门店列表
// @Description 获取销售门店列表（用户公司及子公司）
// @Tags 销售门店
// @Param name query string false "销售门店名称"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDept}} "{"code": 200, "data": [...]}"
// @Router /api/store [get]
// @Security Bearer
func (e StoreController) GetPage(c *gin.Context) {
	s := service.Store{}
	req := dto.StoreGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	p.DeptId = user.GetDeptId(c)
	req.DeptId = user.GetDeptId(c)
	req.Type = model.TypeStore
	list := make([]model.SysDept, 0)
	list, err = s.SetStorePage(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(list, "查询成功")
}

// GetDeliveryPage 获取配送门店列表
// @Summary 获取配送门店列表
// @Description 获取配送门店列表（进入公司后的公司列表）
// @Tags 销售门店
// @Param name query string false "销售门店名称"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDept}} "{"code": 200, "data": [...]}"
// @Router /api/store/delivery [get]
// @Security Bearer
func (e StoreController) GetDeliveryPage(c *gin.Context) {
	s := service.Store{}
	req := dto.StoreGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	req.DeptId = p.DeptId
	req.Type = model.TypeStore
	list := make([]model.SysDept, 0)
	list, err = s.SetStorePage(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(list, "查询成功")
}

// GetAllPage 获取配送门店列表（根公司下所有公司）
// @Summary 获取配送门店列表
// @Description 获取配送门店列表（进入公司后的公司列表）
// @Tags 销售门店
// @Param name query string false "销售门店名称"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDept}} "{"code": 200, "data": [...]}"
// @Router /api/store/delivery [get]
// @Security Bearer
func (e StoreController) GetAllPage(c *gin.Context) {
	s := service.Store{}
	req := dto.StoreGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//var dept model.SysDept
	//数据权限检查
	//err = s.Get(&dto.StoreGetReq{
	//	Id: user.GetDeptId(c),
	//}, &dept)
	//if err != nil {
	//	e.Error(500, err, err.Error())
	//	return
	//}
	//deptId := user.GetDeptId(c)
	//deptIdList := strings.Split(strings.Trim(dept.Path, "/"), "/")
	//if len(deptIdList) > 1 {
	//	deptId, _ = strconv.Atoi(deptIdList[1])
	//}
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	//req.DeptId = deptId
	req.Type = model.TypeStore
	list := make([]model.SysDept, 0)
	//p.DeptId = deptId
	list, err = s.SetStorePage(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(list, "查询成功")
}

// GetAllPage2 获取配送门店列表（根公司下所有公司）
// @Summary 获取配送门店列表
// @Description 获取配送门店列表（进入公司后的公司列表）
// @Tags 销售门店
// @Param name query string false "销售门店名称"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDept}} "{"code": 200, "data": [...]}"
// @Router /api/store/all2 [get]
// @Security Bearer
func (e StoreController) GetAllPageByUser(c *gin.Context) {
	s := service.Store{}
	req := dto.StoreGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var dept model.SysDept
	//数据权限检查
	err = s.Get(&dto.StoreGetReq{
		Id: user.GetDeptId(c),
	}, &dept)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	deptId := user.GetDeptId(c)
	deptIdList := strings.Split(strings.Trim(dept.Path, "/"), "/")
	if len(deptIdList) > 1 {
		deptId, _ = strconv.Atoi(deptIdList[1])
	}
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	req.DeptId = deptId
	req.Type = model.TypeStore
	list := make([]model.SysDept, 0)
	p.DeptId = deptId
	list, err = s.SetStorePage(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(list, "查询成功")
}

// Get 通过id获取销售门店
// @Summary 通过id获取销售门店
// @Description 通过id获取销售门店
// @Tags 销售门店
// @Param id path string true "销售门店id"
// @Success 200 {object} response.Response{data=model.SysDept} "{"code": 200, "data": [...]}"
// @Router /api/store/{id} [get]
// @Security Bearer
func (e StoreController) Get(c *gin.Context) {
	s := service.Store{}
	req := dto.StoreGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.SysDept
	//数据权限检查
	err = s.Get(&req, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加销售门店
// @Summary 添加销售门店
// @Description 添加销售门店
// @Tags 销售门店
// @Accept  application/json
// @Product application/json
// @Param data body dto.StoreInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/store [post]
// @Security Bearer
func (e StoreController) Insert(c *gin.Context) {
	s := service.Store{}
	inspectExpandSvc := service.InspectExpand{}
	req := dto.StoreInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		MakeService(&inspectExpandSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	deptId := user.GetDeptId(c)
	err = s.Insert(&req, deptId)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	err = inspectExpandSvc.Init(user.GetUserId(c), (req.GetId()).(int))
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改销售门店
// @Summary 修改销售门店
// @Description 修改销售门店
// @Tags 销售门店
// @Accept  application/json
// @Product application/json
// @Param data body dto.StoreUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/store [put]
// @Security Bearer
func (e StoreController) Update(c *gin.Context) {
	s := service.Store{}
	req := dto.StoreUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除销售门店
// @Summary 删除销售门店
// @Description 删除销售门店
// @Tags 销售门店
// @Accept  application/json
// @Product application/json
// @Param data body dto.StoreDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/store [delete]
// @Security Bearer
func (e StoreController) Delete(c *gin.Context) {
	s := service.Store{}
	inspectExpandSvc := service.InspectExpand{}
	req := dto.StoreDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		MakeService(&inspectExpandSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var count int64
	err = userSvc.GetCount(&dto.SysUserGetCountReq{DeptIds: []int{req.Id}}, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	if count > 0 {
		e.Error(500, err, "有用户关联，禁止删除！")
		return
	}

	//数据权限检查
	//p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, nil)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	err = inspectExpandSvc.DeleteByDeptId((req.GetId()).(int))
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}

// Enter 进入销售门店
// @Summary 删除销售门店
// @Description 删除销售门店
// @Tags 销售门店
// @Accept  application/json
// @Product application/json
// @Param data body dto.StoreEnterReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/store/enter [post]
// @Security Bearer
func (e StoreController) Enter(c *gin.Context) {
	s := service.Store{}
	req := dto.StoreEnterReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var object model.SysDept
	//数据权限检查
	err = s.Get(&dto.StoreGetReq{
		Id: req.Id,
	}, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	err = e.Cache.Set(model.GetEnterDeptCacheKey(c), req.GetId(), int(config.JwtConfig.Timeout)+7200)
	err = e.Cache.Set(model.GetEnterDeptNameCacheKey(c), object.Name, int(config.JwtConfig.Timeout)+7200)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, "进入失败")
		return
	}
	e.OK(nil, "进入成功")
}
