package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
	"gogs.baozhida.cn/zoie/OAuth-core/sdk/config"
)

type TruckEnterpriseController struct {
	api.Api
}

// GetPage 获取运输企业列表
// @Summary 获取运输企业列表
// @Description 获取运输企业列表
// @Tags 运输企业
// @Param name query string false "运输企业名称"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDept}} "{"code": 200, "data": [...]}"
// @Router /api/truck-enterprise [get]
// @Security Bearer
func (e TruckEnterpriseController) GetPage(c *gin.Context) {
	s := service.TruckEnterprise{}
	req := dto.TruckEnterpriseGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	req.DeptId = user.GetDeptId(c)
	req.Type = model.TypeTruckEnterprise
	list := make([]model.SysDept, 0)
	list, err = s.SetTruckEnterprisePage(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(list, "查询成功")
}

// Get 通过id获取运输企业
// @Summary 通过id获取运输企业
// @Description 通过id获取运输企业
// @Tags 运输企业
// @Param id path string true "运输企业id"
// @Success 200 {object} response.Response{data=model.SysDept} "{"code": 200, "data": [...]}"
// @Router /api/truck-enterprise/{id} [get]
// @Security Bearer
func (e TruckEnterpriseController) Get(c *gin.Context) {
	s := service.TruckEnterprise{}
	req := dto.TruckEnterpriseGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.SysDept
	//数据权限检查
	err = s.Get(&req, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加运输企业
// @Summary 添加运输企业
// @Description 添加运输企业
// @Tags 运输企业
// @Accept  application/json
// @Product application/json
// @Param data body dto.TruckEnterpriseInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/truck-enterprise [post]
// @Security Bearer
func (e TruckEnterpriseController) Insert(c *gin.Context) {
	s := service.TruckEnterprise{}
	req := dto.TruckEnterpriseInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	deptId := user.GetDeptId(c)
	err = s.Insert(&req, deptId)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改运输企业
// @Summary 修改运输企业
// @Description 修改运输企业
// @Tags 运输企业
// @Accept  application/json
// @Product application/json
// @Param id path string true "运输企业id"
// @Param data body dto.TruckEnterpriseUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/truck-enterprise [put]
// @Security Bearer
func (e TruckEnterpriseController) Update(c *gin.Context) {
	s := service.TruckEnterprise{}
	req := dto.TruckEnterpriseUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除运输企业
// @Summary 删除运输企业
// @Description 删除运输企业
// @Tags 运输企业
// @Accept  application/json
// @Product application/json
// @Param data body dto.TruckEnterpriseDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/truck-enterprise [delete]
// @Security Bearer
func (e TruckEnterpriseController) Delete(c *gin.Context) {
	s := service.TruckEnterprise{}
	req := dto.TruckEnterpriseDeleteReq{}
	userSvc := service.SysUser{}
	inspectExpandSvc := service.InspectExpand{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		MakeService(&inspectExpandSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var count int64
	err = userSvc.GetCount(&dto.SysUserGetCountReq{DeptIds: []int{req.Id}}, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	if count > 0 {
		e.Error(500, err, "有用户关联，禁止删除！")
		return
	}

	//数据权限检查
	//p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, nil)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	err = inspectExpandSvc.DeleteByDeptId((req.GetId()).(int))
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}

// Enter 进入运输企业
// @Summary 删除运输企业
// @Description 删除运输企业
// @Tags 运输企业
// @Accept  application/json
// @Product application/json
// @Param data body dto.TruckEnterpriseEnterReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/truck-enterprise/enter [post]
// @Security Bearer
func (e TruckEnterpriseController) Enter(c *gin.Context) {
	s := service.TruckEnterprise{}
	req := dto.TruckEnterpriseEnterReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var object model.SysDept
	//数据权限检查
	err = s.Get(&dto.TruckEnterpriseGetReq{
		Id: req.Id,
	}, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	err = e.Cache.Set(model.GetEnterDeptCacheKey(c), req.GetId(), int(config.JwtConfig.Timeout)+7200)
	err = e.Cache.Set(model.GetEnterDeptNameCacheKey(c), object.Name, int(config.JwtConfig.Timeout)+7200)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, "进入失败")
		return
	}
	e.OK(nil, "进入成功")
}
