package service

import (
	"errors"
	"fmt"
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	cDto "gas-cylinder-api/common/dto"
	"gas-cylinder-api/common/global"
	common "gas-cylinder-api/common/model"
	"gogs.baozhida.cn/zoie/OAuth-core/service"
	"gorm.io/gorm"
	"gorm.io/gorm/utils"
	"time"
)

type Order struct {
	service.Service
}

// GetPage 获取Order列表
func (e *Order) GetPage(c *dto.OrderGetPageReq, list *[]model.Order, count *int64, p *actions.DataPermission) error {
	var err error
	var data model.Order
	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			OrderDeptIdScopes(p.DeptId),
			OrderStateScopes(c.State),
		).
		Preload("Customer").Preload("Store").Preload("User").Preload("Goods").Preload("Spec").
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}

// 创建订单门店和配送门店都可以查看订单
func OrderDeptIdScopes(deptId int) func(db *gorm.DB) *gorm.DB {

	return func(db *gorm.DB) *gorm.DB {
		return db.Where("dept_id = ? or store_id = ?", deptId, deptId)
	}
}

func OrderStateScopes(state int) func(db *gorm.DB) *gorm.DB {

	return func(db *gorm.DB) *gorm.DB {
		if state == 0 {
			return db
		}
		if state == model.OrderStateInDelivery {
			return db.Where("state = ? or state = ?", model.OrderStateInDelivery, model.OrderStateInDeliveryInspected)
		}
		return db.Where("state = ? ", state)
	}
}
func (e *Order) GetDeliveryPage(c *dto.OrderGetDeliveryPageReq, list *[]model.Order, count *int64, p *actions.DataPermission) error {
	var err error
	var data model.Order
	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			OrderStateScopes(c.State),
		).
		Where("user_id = ? and state > 1", p.UserId).
		Preload("Customer").Preload("Store").Preload("Goods").Preload("Spec").
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}

// Get 获取Order对象
func (e *Order) Get(d *dto.OrderGetReq, orderModel *model.Order, p *actions.DataPermission) error {
	err := e.Orm.
		Scopes(actions.Permission(orderModel.TableName(), p)).
		First(orderModel, d.GetId()).Error

	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.GetFailedErr
	}

	return nil
}

// Insert 创建Order对象
func (e *Order) Insert(c *dto.OrderInsertReq) error {
	var err error
	var data model.Order

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	// 添加订单
	c.Generate(&data)
	err = tx.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	c.Id = data.Id
	return nil

}
func (e *Order) AppletInsert(c *dto.OrderAppletInsertReq) error {
	var err error
	var data model.Order

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()
	// 查询小程序客户是否存在
	storeCustomerId := c.CustomerId + "_" + utils.ToString(c.DeptId)
	var customer model.Customer
	err = e.Orm.
		Where("id = ?", c.CustomerId).
		First(&customer).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}

	store, err := model.GetProvCodeById(c.StoreId)
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}

	// 查询门店客户是否存在
	var storeCustomer model.Customer

	err = e.Orm.
		Where("id = ?", storeCustomerId).
		First(&storeCustomer).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			// TODO 1.1.1.16新增客户信息
			// 添加客户
			obj := model.Customer{
				ProvCustomer: model.ProvCustomer{
					Id:             storeCustomerId,
					PrincipalName:  customer.Name,
					PrincipalPhone: customer.PrincipalPhone,
					ShopName:       customer.Name,
					Address:        c.Address,
					AddressImg:     c.AddressImg,
					Lng:            c.Lng,
					Lat:            c.Lat,
					Type:           customer.Type,
					City:           c.City,
					Area:           c.Area,
					Remark:         c.Remark,
					StoreCode:      store.CmpCode,
				},
				Name: customer.Name,
				DeptBy: common.DeptBy{
					DeptId: c.StoreId,
				},
			}
			dataIntegrity := CustomerCheckDataIntegrity(obj)
			if dataIntegrity {
				// TODO 同步省平台
				obj.IsSyncProv = true
			}
			err = tx.Create(&obj).Error
			if err != nil {
				e.Log.Errorf("db error: %s", err)
				return global.CreateFailedErr
			}
		} else {
			return global.CreateFailedErr
		}
	}

	if len(storeCustomer.ProvCustomer.Id) > 0 {
		storeCustomer.Name = c.Name
		storeCustomer.Lng = c.Lng
		storeCustomer.Lat = c.Lat
		storeCustomer.City = c.City
		storeCustomer.Area = c.Area
		storeCustomer.Address = c.Address
		storeCustomer.AddressImg = c.AddressImg

		dataIntegrity := CustomerCheckDataIntegrity(storeCustomer)
		if dataIntegrity {
			// TODO 同步省平台 更新客户信息
			storeCustomer.IsSyncProv = true
		}
		err = tx.Save(&storeCustomer).Error
		if err != nil {
			e.Log.Errorf("db error: %s", err)
			return global.UpdateFailedErr
		}
	}

	// 添加订单
	c.Generate(&data)
	data.CustomerId = storeCustomerId
	err = tx.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	c.Id = data.Id
	return nil

}

// Update 修改Order对象
func (e *Order) Update(c *dto.OrderUpdateReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var orderModel = model.Order{}
	// 查询订单是否存在
	err = e.Orm.Scopes(actions.Permission(orderModel.TableName(), p)).
		First(&orderModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	// 订单为未派送才可以取消

	c.Generate(&orderModel)
	err = tx.Save(&orderModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	c.Id = orderModel.Id

	return nil
}
func (e *Order) AppletUpdate(c *dto.OrderAppletUpdateReq) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var orderModel = model.Order{}
	// 查询订单是否存在
	err = e.Orm.First(&orderModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	// 如果修改地址 则同步修改用户信息
	if orderModel.Address != c.Address {
		// TODO 同步省平台 修改用户信息
		err = tx.Model(&model.Customer{}).Where("id = ?", orderModel.CustomerId).Updates(
			map[string]interface{}{
				"area":    c.Area,
				"city":    c.City,
				"address": c.Address,
				"name":    c.Name,
			}).Error
		if err != nil {
			return global.UpdateFailedErr
		}
	}

	// 订单为未派送才可以修改
	if orderModel.State != model.OrderStateOrder {
		return errors.New(fmt.Sprintf("订单状态为%s状态，无法修改", model.OrderStateMap[orderModel.State]))
	}
	c.Generate(&orderModel)
	err = tx.Save(&orderModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	c.Id = orderModel.Id

	return nil
}

// Remove 删除Order
func (e *Order) Remove(c *dto.OrderDeleteReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var orderModel model.Order

	// 查询订单是否存在
	err = e.Orm.Scopes(actions.Permission(orderModel.TableName(), p)).
		First(&orderModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return nil
		}
		return global.DeleteFailedErr
	}
	if orderModel.State != model.OrderStateOrder && orderModel.State != model.OrderStateCancel {
		err = errors.New(fmt.Sprintf("订单状态为%s，无法删除", model.OrderStateMap[orderModel.State]))
		return err
	}
	db := tx.Delete(&orderModel)

	if err = db.Error; err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.DeleteFailedErr
	}
	if db.RowsAffected == 0 {
		return global.DeleteNotFoundOrNoPermissionErr
	}

	return nil
}
func (e *Order) Cancel(c *dto.OrderCancelReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var orderModel model.Order

	// 查询订单是否存在
	err = e.Orm.Scopes(actions.Permission(orderModel.TableName(), p)).
		First(&orderModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	if orderModel.State == model.OrderStateCancel {
		return nil
	}

	if orderModel.State != model.OrderStateOrder && orderModel.State != model.OrderStateDelivery {
		return errors.New(fmt.Sprintf("订单状态为%s，禁止取消", model.OrderStateMap[orderModel.State]))
	}

	orderModel.State = model.OrderStateCancel
	orderModel.CancelTime = common.Time(time.Now())
	db := tx.Save(&orderModel)

	if err = db.Error; err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}
	if db.RowsAffected == 0 {
		return global.UpdateNotFoundOrNoPermissionErr
	}

	return nil
}

// Delivery 派单
func (e *Order) Delivery(c *dto.OrderDeliveryReq, p *actions.DataPermission) error {
	var err error

	var orderModel = model.Order{}
	// 查询订单是否存在
	err = e.Orm.Scopes(OrderDeptIdScopes(p.DeptId)).
		First(&orderModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	c.Generate(&orderModel)
	err = e.Orm.Save(&orderModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	c.Id = orderModel.Id

	return nil
}

// UpdateState 修改订单状态
func (e *Order) UpdateState(c *dto.OrderUpdateStateReq, p *actions.DataPermission) error {
	var err error

	var orderModel = model.Order{}
	// 查询订单是否存在
	err = e.Orm.
		Scopes(InDeptIdScopes(p.DeptId)).
		Preload("User").Preload("Store").Preload("Dept").
		First(&orderModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	// 配送中
	if c.State == model.OrderStateInDelivery {
		orderModel.State = c.State
		orderModel.DeliveryTime = common.Time(time.Now())
	}
	// 已送达
	if c.State == model.OrderStateArrive {
		orderModel.State = c.State
		orderModel.ArriveTime = common.Time(time.Now())
		err = InsertOperationLogByOptType26Or36(tx, "26", c.ChipUid, orderModel)
		if err != nil {
			e.Log.Errorf("db error: %s", err)
			return global.UpdateFailedErr
		}

		var orderInnerCodeList []model.OrderInnerCode
		for _, chipUid := range c.ChipUid {
			orderInnerCodeList = append(orderInnerCodeList, model.OrderInnerCode{
				InnerCode: chipUid,
				State:     1,
			})
		}
		orderModel.OrderInnerCodeList = orderInnerCodeList
	}
	// 已取消
	if c.State == model.OrderStateCancel && orderModel.State == model.OrderStateArrive {
		orderModel.State = c.State
		orderModel.CancelTime = common.Time(time.Now())
		// 已取消钢瓶不参与流传
		var orderInnerCodeMap = make(map[string]int)
		for _, v := range orderModel.OrderInnerCodeList {
			orderInnerCodeMap[v.InnerCode] = v.State
		}
		var chipUid []string
		for i := 0; i < len(c.ChipUid); i++ {
			if state, ok := orderInnerCodeMap[c.ChipUid[i]]; ok && state == 1 {
				chipUid = append(chipUid, c.ChipUid[i])
			}
		}
		err = InsertOperationLogByOptType26Or36(tx, "36", chipUid, orderModel)
		if err != nil {
			e.Log.Errorf("db error: %s", err)
			return global.UpdateFailedErr
		}
		// 更新钢瓶状态为已取消
		var chipUidMap = make(map[string]struct{})
		for _, s := range c.ChipUid {
			chipUidMap[s] = struct{}{}
		}

		for i := 0; i < len(orderModel.OrderInnerCodeList); i++ {
			if _, ok := chipUidMap[orderModel.OrderInnerCodeList[i].InnerCode]; ok {
				orderModel.OrderInnerCodeList[i].State = 2
			}
		}
		// 判断所有钢瓶状态是否为已取消
		// 所有钢瓶状态为已取消，订单状态改为已取消，否则还是已送达
		var flag = true
		for _, code := range orderModel.OrderInnerCodeList {
			if code.State == 1 {
				flag = false
				break
			}
		}
		// 所有钢瓶状态为已取消，订单状态改为已取消，否则还是已送达
		if !flag {
			orderModel.State = model.OrderStateArrive
		}

	}

	err = e.Orm.Model(&orderModel).Updates(&orderModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	c.Id = orderModel.Id

	return nil
}

func AppletOrderState(state int) func(db *gorm.DB) *gorm.DB {

	return func(db *gorm.DB) *gorm.DB {
		if state == 0 {
			return db
		}
		if state == 1 || state == 2 {
			return db.Where("state = 1 or state = 2 ")
		}
		return db.Where("state = ? ", state)
	}
}

func (e *Order) AppletGetPage(c *dto.AppletOrderGetPageReq, list *[]model.Order, count *int64) error {
	var err error
	var data model.Order
	err = e.Orm.Model(&data).
		Scopes(
			AppletOrderState(c.State),
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
		).
		Preload("Customer").Preload("Store").Preload("User").Preload("Goods").Preload("Spec").
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}
